package linearalgebra;

import linearalgebra.exception.UnboundValueError;

public final class UnknownValue implements Cloneable {
	private static final int FREE = 0;
	private static final int BOUND = 1;
	private static final int CONSTRAINT = 2;
	
	private int m_state = FREE;
	private String m_name;
	private double m_value;
	private LinearExpression m_constraint = null;
	
	public UnknownValue( String name ) {
		m_name = name;
	}
	
	public void resolveBoundConstraints() {
		if( m_constraint != null ) {
			m_constraint.resolveBoundUnknowns();
			setConstraint(m_constraint);
		}
	}
	
	public String getName() {
		return m_name;
	}
	
	public Object clone() {
		try {
			return super.clone();
		} catch( CloneNotSupportedException e ) {
			throw new InternalError("UnknownValue.clone() should not throw CloneNotSupportedException.");
		}
	}
	
	public boolean isBound() {
		return m_state == BOUND;
	}
	
	public boolean isFree() {
		return m_state == FREE;
	}
	
	public boolean isConstrained() {
		return m_state == CONSTRAINT;
	}
	
	public double getNumericValue() throws UnboundValueError {
		if( isBound() ) {
			return m_value;
		}
		throw new UnboundValueError();
	}
	
	public void bindUnknown( double value ) {
		m_state = BOUND;
		m_value = value;
		m_constraint = null;
	}
	
	public void setConstraint( LinearExpression constraint ) {
		if( constraint.getVariableCount() == 0 ) {
			bindUnknown(constraint.getConstant());
		} else {
			m_state = CONSTRAINT;
			m_constraint = constraint;
		}
	}
	
	public LinearExpression getConstraint() {
		return m_constraint;
	}

	public void clearBindings() {
		m_state = FREE;
		m_constraint = null;
	}
	
	public void solveFor( double y ) {
		bindUnknown(y);
	}
	
	public String getDescription() {
		StringBuffer rc = new StringBuffer();
		rc.append( getName() );
		switch( m_state ) {
			case FREE: {
				rc.append( " is unbound." );
			} break;
			case BOUND: {
				rc.append( " = " );
				rc.append( m_value );
			} break;
			case CONSTRAINT: {
				rc.append( " = " );
				rc.append( m_constraint );
			} break;
		}
		return rc.toString();
	}
	
	public String toString() {
		return getName();
	}
	
	public void substituteConstraints() {
		if( m_constraint != null ) {
			m_constraint.substituteConstraint(this);
		}
	}
}
