package linearalgebra;

import linearalgebra.numeric.*;
import linearalgebra.exception.*;

public class VectorSpace implements linearalgebra.VectorSpaceInterface {
	private NumericMatrixInterface	m;

	public VectorSpace(NumericMatrixInterface m) {
		this.m = m;
	}

	public VectorSpace(NumericVectorInterface[] columns) {
		double[][] va = new double[columns.length][];
		for(int i = 0; i < columns.length; i++) {
			va[i] = columns[i].toArray();
		}
		m = new TransposeWrapper(new Matrix(va));
	}

	public NumericMatrixInterface getMatrix() {
		return (NumericMatrixInterface) m.clone();
	}

	public boolean areIndependentColumns() {
		return getRank() == m.getRowCount();
	}

	public boolean areOrthogonalColumns() {
		for(int i = 0; i < getColumnCount(); i++) {
			VectorInterface c1 = getColumnVector(i);
			for(int j = i + 1; j < getColumnCount(); j++) {
				VectorInterface c2 = getColumnVector(j);
				if(!c1.isOrthogonal(c2)) {
					return false;
				}
			}
		}
		return true;
	}

	public boolean normalizeColumns() {
		boolean rc = true;
		for(int i = 0; i < getColumnCount(); i++) {
			VectorInterface col = getColumnVector(i);
			rc = rc && col.normalize();
		}
		return rc;
	}
	private int[]	m_basisColumns	= null;

	public int[] getBasisColumns() {
		if(m_basisColumns == null) {
			NumericMatrixInterface u = getU();
			int[] rows = getBasisRows();
			m_basisColumns = new int[rows.length];
			for(int i = 0; i < rows.length; i++) {
				m_basisColumns[i] = u.getRowLeftPivotIndex(rows[i]);
			}
		}
		return m_basisColumns;
	}
	private int[]	m_basisRows	= null;

	public int[] getBasisRows() {
		if(m_basisRows == null) {
			NumericMatrixInterface m2 = getU();
			int[] basisRows = new int[m2.getRowCount()];
			int rank = 0;
			for(int row = 0; row < m2.getRowCount(); row++) {
				try {
					m2.getRowLeftPivotIndex(row);
					basisRows[rank++] = row;
				} catch(NoPivotError e) {
				}
			}
			m_basisRows = new int[rank];
			System.arraycopy(basisRows, 0, m_basisRows, 0, rank);
		}
		return m_basisRows;
	}

	private NumericMatrixInterface getMatrixRows(NumericMatrixInterface m2, int[] rowList) {
		double[][] av = new double[rowList.length][m2.getColumnCount()];
		for(int row = 0; row < av.length; row++) {
			for(int col = 0; col < av[row].length; col++) {
				av[row][col] = m2.getElement(rowList[row], col);
			}
		}
		return new Matrix(av);
	}

	private NumericMatrixInterface getMatrixColumns(NumericMatrixInterface m2, int[] colList) {
		double[][] av = new double[m2.getRowCount()][colList.length];
		for(int col = 0; col < colList.length; col++) {
			for(int row = 0; row < m2.getRowCount(); row++) {
				av[row][col] = m2.getElement(row, colList[col]);
			}
		}
		return new Matrix(av);
	}

	public int getColumnCount() {
		return m.getColumnCount();
	}

	public VectorInterface getColumnVector(int colIndex) {
		return m.getNumericColumnView(colIndex).getVectorView();
	}
	private int	m_nRank	= -1;

	public int getRank() {
		if(m_nRank < 0) {
			getNullspace();
		}
		return m_nRank;
	}
	private int	m_nDimension	= -1;

	public int getDimension() {
		if(m_nDimension < 0) {
			getBasisColumns();
		}
		return m_nDimension;
	}

	public VectorSpaceInterface getSubspace(VectorInterface v) {
		// This method is derived from interface linearalgebra.VectorSpaceInterface
		// to do: code goes here
		return null;
	}

	public boolean isInconsistent() {
		// This method is derived from interface linearalgebra.VectorSpaceInterface
		// to do: code goes here
		return false;
	}

	public boolean isMember(NumericVectorInterface v) {
		LinearSystem s = new LinearSystem((NumericMatrixInterface) m.clone(), new NumericColumn(v), false);
		s.solve();
		return s.isNonsingular();
	}

	public boolean isNonsingular() {
		// This method is derived from interface linearalgebra.VectorSpaceInterface
		// to do: code goes here
		return false;
	}

	public boolean isSubspace(VectorSpaceInterface v) {
		// This method is derived from interface linearalgebra.VectorSpaceInterface
		// to do: code goes here
		return false;
	}

	public boolean isUnderdetermined() {
		// This method is derived from interface linearalgebra.VectorSpaceInterface
		// to do: code goes here
		return false;
	}

	public boolean spansSpace(VectorSpaceInterface v) {
		// This method is derived from interface linearalgebra.VectorSpaceInterface
		// to do: code goes here
		return false;
	}

	public final VectorSpaceInterface getRowSpace() {
		// get a copy of the matrix
		// get the row basis of the matrix, changing the copy in the process
		int[] basisRows = getBasisRows();
		// return correct rows of original matrix
		// we could also return the rows of U since basic operations do not change the row space
		return new VectorSpace(getMatrixRows(m, basisRows));
	}

	public VectorSpaceInterface getNullspace() {
		NumericMatrixInterface u = getU();
		u.swapRows(p);
		LinearSystem s = new LinearSystem(u, new NumericColumn(m.getRowCount()), false);
		s.solve();
		m_nRank = s.getRank();
		VectorSpaceInterface rc = new VectorSpace(s.getGeneralSolution().getMatrixView());
		u.swapRows(p);
		return rc;
	}

	public VectorSpaceInterface getColumnSpace() {
		int[] basisCols = getBasisColumns();
		// we must return the columns of the original matrix
		// column space is not preserved during transpose/row elimination operations
		return new VectorSpace(getMatrixColumns(m, basisCols));
	}

	public VectorSpaceInterface getLeftNullspace() {
		// find the number of zero rows in U (getRowCount()-getRank())
		// the last number of rows of inverseL
		int r = m.getRowCount() - getRank();
		int[] basisRows = new int[r];
		NumericMatrixInterface inverseL = getInverseL();
		for(int i = 0; i < basisRows.length; i++) {
			basisRows[i] = inverseL.getRowCount() - r + i;
		}
		return new VectorSpace(new TransposeWrapper(getMatrixRows(inverseL, basisRows)));
	}

	public String toString() {
		return m.toString();
	}
	private NumericMatrixInterface	L	= null;
	private NumericMatrixInterface	U	= null;
	private PermutationMatrix		p;

	private NumericMatrixInterface getU() {
		if(U == null) {
			U = (NumericMatrixInterface) m.clone();
			p = new PermutationMatrix(U.getRowCount());
			L = (NumericMatrixInterface) U.factorL(p, false);
			// swap rows back to match rows with original matrix
			U.swapRows(p);
		}
		return U;
	}

	private PermutationMatrix getP() {
		if(p == null) {
			getU();
		}
		return p;
	}

	private NumericMatrixInterface getL() {
		if(L == null) {
			getU();
		}
		return L;
	}
	private NumericMatrixInterface	invL	= null;

	private NumericMatrixInterface getInverseL() {
		if(invL == null) {
			try {
				invL = (NumericMatrixInterface) getL().factorInverse();
			} catch(UninvertibleMatrixException e) {
				throw new Error();
			}
			// getting the inverse destroys the original matrix, so get rid of it.
			// we really don't need L, this is a just in time calculation of inverse L
			// so we don't clone if for the inverse, we just drop it.
			L = null;
			// we need to swap columns to get correct row vectors
			// we need to swap the columns of inverse L
			// A x B = Bt x At
			TransposeWrapper t = new TransposeWrapper(invL);
			t.swapRows((PermutationMatrix) getP().getTranspose());
		}
		return invL;
	}

	public boolean hasExistence() {
		boolean rc = false;
		if(m.getRowCount() <= m.getColumnCount()) {
			if(getRank() == m.getRowCount()) {
				rc = true;
			}
		}
		return rc;
	}

	public boolean hasUniqueness() {
		boolean rc = false;
		if(m.getRowCount() >= m.getColumnCount()) {
			if(getRank() == m.getColumnCount()) {
				rc = true;
			}
		}
		return rc;
	}

	public boolean isOrthogonal(VectorSpaceInterface vs) {
		for(int i = 0; i < getColumnCount(); i++) {
			VectorInterface c1 = getColumnVector(i);
			for(int j = 0; j < vs.getColumnCount(); j++) {
				VectorInterface c2 = vs.getColumnVector(i);
				if(!c1.isOrthogonal(c2)) {
					return false;
				}
			}
		}
		return true;
	}
}