package linearalgebra;

import linearalgebra.numeric.NumericVectorInterface;
import linearalgebra.numeric.NumericMatrixInterface;

public interface VectorSpaceInterface {
    public NumericMatrixInterface getMatrix();
    
	public VectorSpaceInterface getSubspace( VectorInterface v );

	/**
	 * Checks if all of the vectors in v can be described by a linear combination
	 * of the vectors in this space. If so, then v spans this.
	 * 
	 * @param v A vector space that may span this space.
	 * @return Returns true if v spans this space. Returns false otherwise.
	 */
	public boolean spansSpace( VectorSpaceInterface v );

	/**
	 * The space is defined by the system ax=b. Returns true if a solution
	 * exists for ax=v meaning v is located in the hyper-plane defined by this space. 
	 * Returns false otherwise.
	 * 
	 * @param v The vector to check for membership.
	 */
	public boolean isMember( NumericVectorInterface v );

	/**
	 * Returns true if all vectors in v are contained in this space.
	 * 
	 * @param v
	 */
	public boolean isSubspace( VectorSpaceInterface v );
	
	/**
	 * Returns the number of non-zero pivots in U after the system A 
	 * describing the vector space has been factored into A=LU.
	 */
	public int getRank();
	
	/** 
	 * All bases of this space have the same number of vectors. This number
	 * is called the degree of freedom, or the dimension, of the space.
	 * Note that this is a different concept than the dimension of a vector.
	 */
	public int getDimension();

	/**
	 * Returns the number of column vectors used to define this space. This number may
	 * change as the space is manipulated.
	 */
	public int getColumnCount();

	/**
	 * Returns one of the column vectors of the space.
	 * 
	 * @param colIndex
	 */
	public VectorInterface getColumnVector(int colIndex);

	/**
	 * For the system ax=b that describes the space, return true if a!=0. There exists
	 * a single solution to ax=b. Returns false otherwise.
	 */
	public boolean isNonsingular();

	/**
	 * For the system ax=b describing this space, if a=0 and b=0 there are an infinite
	 * number of solutions. Any x satisfies 0x=0.
	 */
	public boolean isUnderdetermined();

	/**
	 * For the system ax=b describing this space, if a=0 and b!=0 there are an zero
	 * number of solutions. No x satisfies 0x=b.
	 */
	public boolean isInconsistent();
	
	/**
	 *	Returns true if all non-trivial combinations of all column vectors
	 *	are non-zero.
	 */
	public boolean areIndependentColumns();
	public boolean areOrthogonalColumns();
	public boolean normalizeColumns();
	
	/**
	 * A basis for a vector space is the set of column vectors such that all
	 * vectors are linearly independent and the set spans the space.
	 */
	public VectorSpaceInterface getColumnSpace();
	
	/**
	 *	Returns the space spanned by all of the rows
	 */
	public VectorSpaceInterface getRowSpace();
	
	
	/**
	 * Returns the columns for the column basis
	 */
	public int[] getBasisColumns();

	public VectorSpaceInterface getNullspace();
	public VectorSpaceInterface getLeftNullspace();
	
	/**
	 * There is 1 to infinite solutions.
	 * At least one solution.
	 */
	public boolean hasExistence();
	
	/**
	 * There is 0 or 1 solution
	 * At most one solution.
	 */
	public boolean hasUniqueness();
	
	public boolean isOrthogonal( VectorSpaceInterface vs );
}