package linearalgebra.numeric;

import linearalgebra.PermutationMatrix;
import linearalgebra.MatrixInterface;
import linearalgebra.ColumnInterface;
import linearalgebra.RowInterface;
import linearalgebra.Util;
import linearalgebra.exception.SizeMismatchError;
import linearalgebra.VectorInterface;

public abstract class AbstractMatrix implements linearalgebra.numeric.NumericMatrixInterface {
	private int m_rowCount = 0;
	private int m_colCount = 0;
	protected boolean m_bFrontPivot = true;

	public AbstractMatrix( int rows, int cols ) {
		m_rowCount = rows;
		m_colCount = cols;
		pivotIndexForL();
	}
	
	public abstract void setElement( int row, int col, double element );	
	public abstract void scaleElement( int row, int col, double scalar );	
	public abstract void incrementElement( int row, int col, double value );	
	
	public Object clone() {
		try {
			return super.clone();
		} catch( CloneNotSupportedException e ) {
			throw new InternalError();
		}
	}
	
	public final int getRowCount() {
		return m_rowCount;
	}
	
	public final int getColumnCount() {
		return m_colCount;
	}
	
	public void swapRows( int row1, int row2 ) {
		for( int col = 0; col < getColumnCount(); col++ ) {
			double temp = getElement(row1, col);
			setElement(row1, col, getElement(row2, col) );
			setElement(row2, col, temp );
		}
	}
	
	public boolean hasRowPivot( int row ) {
		for( int col = 0; col < getColumnCount(); ++col ) {
			if( getElement(row, col) != 0 ) {
				return true;
			}
		}
		return false;
	}
	
	public boolean hasColumnPivot( int col ) {
		for( int row = 0; row < getRowCount(); ++row ) {
			if( getElement(row, col) != 0 ) {
				return true;
			}
		}
		return false;
	}
	
	public int getRowLeftPivotIndex( int row ) {
		for( int col = 0; col < getColumnCount(); ++col ) {
			if( getElement(row, col) != 0 ) {
				return col;
			}
		}
		throw new linearalgebra.exception.NoPivotError();
	}
	
	public int getRowRightPivotIndex( int row ) {
		for( int col = getColumnCount()-1; col >= 0; --col ) {
			if( getElement(row, col) != 0 ) {
				return col;
			}
		}
		throw new linearalgebra.exception.NoPivotError();
	}
	
	public int getColumnTopPivotIndex( int col ) {
		for( int row = 0; row < getRowCount(); ++row ) {
			if( getElement(row, col) != 0 ) {
				return row;
			}
		}
		throw new linearalgebra.exception.NoPivotError();
	}
	
	public int getColumnBottomPivotIndex( int col ) {
		for( int row = getRowCount()-1; row >= 0 ; --row ) {
			if( getElement(row, col) != 0 ) {
				return row;
			}
		}
		throw new linearalgebra.exception.NoPivotError();
	}
	
	private int getRowPivotIndex( int row ) {
		if( m_bFrontPivot ) {
			int pivot = getRowLeftPivotIndex(row);
			if( pivot < row ) {
				// pivot is in L - should not be
				// try other direction
				m_bFrontPivot = false;
				pivot = getRowRightPivotIndex(row);
			}
			return pivot;
		} else {
			int pivot = getRowRightPivotIndex(row);
			if( pivot > row ) {
				// pivot is in U - should not be
				// try other direction
				m_bFrontPivot = true;
				pivot = getRowLeftPivotIndex(row);
			}
			return pivot;
		}
	}
	
	public void multiplyRow( int row, double scalar ) {
		for( int i = 0; i < getColumnCount(); i++ ) {
			scaleElement( row, i, scalar );
		}
	}
	
	public void combineRows( int srcRow, int dstRow, double scalar ) {
		for( int i = 0; i < getColumnCount(); i++ ) {
			incrementElement(dstRow, i, scalar*getElement(srcRow, i));
		}
	}
	
	public final NumericMatrixInterface plus( NumericMatrixInterface m ) {
		NumericMatrixInterface mm = (NumericMatrixInterface)clone();
		return mm.plusEquals(m);
	}
	
	public final NumericMatrixInterface multiply( double scalar ) {
		NumericMatrixInterface mm = (NumericMatrixInterface)clone();
		return mm.multiplyEquals(scalar);
	}
		
	public NumericMatrixInterface plusEquals( NumericMatrixInterface m ) {
		if( getRowCount() != m.getRowCount() || getColumnCount() != m.getColumnCount() ) {
			throw new SizeMismatchError("Matrices must have similar dimensions for addition.");
		}
		for( int i = 0; i < getRowCount(); i++ ) {
			for( int j = 0; j < getColumnCount(); j++ ) {
				incrementElement( i, j, m.getElement(i, j) );
			}
		}
		return this;
	}
	
	public NumericMatrixInterface multiplyEquals( double scalar ) {
		for( int i = 0; i < getRowCount(); i++ ) {
			for( int j = 0; j < getColumnCount(); j++ ) {
				scaleElement( i, j, scalar );
			}
		}
		return this;
	}

	public NumericMatrixInterface product( NumericMatrixInterface right ) {
		if( getColumnCount() != right.getRowCount() ) {
			throw new SizeMismatchError("Row count of first matrix must equal column count of second matrix for cross product.");
		}
		Matrix rc = new Matrix( getRowCount(), right.getColumnCount() );
		
/*		
		for( int row = 0; row < rc.getRowCount(); row++ ) {
			for( int col = 0; col < rc.getColumnCount(); col++ ) {
				rc.setElement(row, col, 0.0);
				for( int k = 0; k < getColumnCount(); k++ ) {
					rc.setElement(row, col, rc.getElement(row, col)+getElement(row, k)*right.getElement(k, col));
				}
			}
		}		
*/		
		// this is an alternative way to calculate cross product by summing columns
		// column n of the result is a combination of all columns on the left weighted
		// by the values in column n on the right
		for( int i = 0; i < getColumnCount(); i++ ) {
			NumericColumnInterface leftColumn = getNumericColumnView(i);
			for( int j = 0; j < right.getColumnCount(); j++ ) {
				rc.getNumericColumnView(j).plusEquals(leftColumn.multiply(right.getElement(i,j)) );
			}
		}
		return rc;
	}

	public NumericMatrixInterface power( int power ) {
		NumericMatrixInterface rc = this;
		for( int i = 1; i < power; i++ ) {
			rc = rc.product(this);
		}
		return rc;
	}
	
	public MatrixInterface factorL( PermutationMatrix p ) {
		return factorL( p, true );
	}
	
	private boolean setupColumnPivot( int row, int col, PermutationMatrix p, MatrixInterface l, boolean partialPivot ) {
		boolean rc = true;
		if( partialPivot ) {
			// do partial pivot swap
			rc = partialPivotDown( row, col, p, l );
		} else {
			// locate next pivot
			int pivotRow = row;
			while( Util.isZero( getElement( pivotRow, col ) ) ) {
				pivotRow++;
				if( pivotRow >= getRowCount() ) {
					// remaining elements in column are zero
					return false;
				}
			}
			if( row != pivotRow ) {
				swapRows(col, pivotRow);
				p.swapRows(col, pivotRow);
				l.swapRows(col, pivotRow);
			}
		}
		return rc;
	}
	
	public void eliminateRowsBeneath( int rowStart, int colStart, PermutationMatrix p, NumericMatrixInterface l ) {
		for( int row = rowStart+1; row < getRowCount(); row++ ) {
			double pivotValue = getElement(row, colStart) / getElement( rowStart, colStart );
			combineRows( rowStart, row, -pivotValue );
			l.setElement( row, rowStart, pivotValue );
		}
	}
	
	public MatrixInterface factorL(PermutationMatrix p, boolean partialPivot) {
		Matrix rc = new Matrix(getRowCount(), getRowCount());
		
		int row = 0;
		int col = 0;
		final int maxRows = getRowCount()-1;
		while( row < maxRows && col < getColumnCount() ) {
			if( setupColumnPivot( row, col, p, rc, partialPivot ) ) {
				eliminateRowsBeneath( row, col, p, rc );
				// move to next column and to next row
				row++;
				col++;
			} else {
				// rest of column is zero
				// do next column
				// but stay on same row
				col++;
			}
		}

		setDiagonalToOne(rc);
		return rc;
	}
	
	public MatrixInterface factorU(PermutationMatrix p) {
		Matrix rc = new Matrix(getColumnCount(), getColumnCount());
		rc.pivotIndexForU();
		for( int col = getColumnCount()-1; col >= 0; col-- ) {
			int pivotIndex = col;
			while( getElement( pivotIndex, col ) == 0.0 ) {
				pivotIndex--;
				if( pivotIndex < 0 ) {
					// no pivot for this column
					col--;
					pivotIndex = col;
				}
				if( col < 0 ) {
					// remaining elements are zero
					setDiagonalToOne(rc);
					return rc;
				}
			}
			if( col != pivotIndex ) {
				swapRows(col, pivotIndex);
				p.swapRows(col, pivotIndex);
				rc.swapRows(col, pivotIndex);
			}
			for( int row = col-1; row >= 0; row-- ) {
				double m = getElement(row, col) / getElement( col, col );
				combineRows( col, row, -m );
				rc.setElement( row, col, m );
			}
		}
		setDiagonalToOne(rc);
		return rc;
	}
	
	public MatrixInterface factorInverse() throws linearalgebra.exception.UninvertibleMatrixException {
		if( getRowCount() != getColumnCount() ) {
			throw new linearalgebra.exception.UninvertibleMatrixException();
		}
		Matrix rc = new Matrix( getRowCount() );		
		for( int row = 0; row < getRowCount(); row++ ) {
			double pivotValue = getElement(row, row);
			if( Util.isZero( pivotValue ) ) {
				throw new linearalgebra.exception.UninvertibleMatrixException();
			}
			for( int zeroRow = row+1; zeroRow < getRowCount(); zeroRow++ ) {
				double zeroValue = -getElement(zeroRow, row);
				if( !Util.isZero(zeroValue) ) {
					zeroValue /= pivotValue;
					combineRows(row, zeroRow, zeroValue);
					rc.combineRows(row, zeroRow, zeroValue);
					setElement( zeroRow, row, 0.0 );
				}
			}
		}
		for( int row = getRowCount()-1; row >= 0; row-- ) {
			double pivotValue = getElement(row, row);
			if( Util.isZero( pivotValue ) ) {
				throw new linearalgebra.exception.UninvertibleMatrixException();
			}
			for( int zeroRow = row-1; zeroRow >= 0; zeroRow-- ) {
				double zeroValue = -getElement(zeroRow, row);
				if( !Util.isZero(zeroValue) ) {
					zeroValue /= pivotValue;
					combineRows(row, zeroRow, zeroValue);
					rc.combineRows(row, zeroRow, zeroValue);
					setElement( zeroRow, row, 0.0 );
				}
			}
		}
		for( int row = 0; row < getRowCount(); row++ ) {
			double pivot = getElement(row, row);
			if( Util.isZero( pivot ) ) {
				throw new linearalgebra.exception.UninvertibleMatrixException();
			}
			setElement( row, row, 1.0 );
			rc.multiplyRow(row, 1.0/pivot);
		}
		return rc;
	}

	public boolean isIdentity() {
		for( int row = 0; row < getRowCount(); row++ ) {
			for( int col = 0; col < getColumnCount(); col++ ) {
				if( row == col ) {
					if( !Util.isZero(getElement(row, col)-1.0) ) {	// close enough
						return false;
					}
				} else {
					if( !Util.isZero(getElement(row, col)) ) {	// close enough
						return false;
					}
				}
			}
		}
		return true;
	}
	
	public boolean isUpperRightTriangular() { 
		for( int row = 0; row < getRowCount(); row++ ) {
			for( int col = 0; col < getColumnCount(); col++ ) {
				if( row > col ) {
					if( getElement(row, col) != 0.0 ) {
						return false;
					}
				}
			}
		}
		return true;
    }
    
	public boolean isLowerLeftTriangular() {
		for( int row = 0; row < getRowCount(); row++ ) {
			for( int col = 0; col < getColumnCount(); col++ ) {
				if( row < col ) {
					if( getElement(row, col) != 0.0 ) {
						return false;
					}
				}
			}
		}
		return true;
    }
    
	public boolean isSymmetric() {
		if( getRowCount() != getColumnCount() ) {
			return false;
		}
		for( int row = 1; row < getRowCount(); row++ ) {
			for( int col = 0; col < getColumnCount(); col++ ) {
				if( !Util.isZero(getElement( row, col ) - getElement( col, row ) ) ) { 
					return false;
				}
			}
		}
		return true;
	}
    
	public boolean isPositiveDefinite() {
		if( getRowCount() != getColumnCount() ) {
			return false;
		}
		for( int row = 1; row < getRowCount(); row++ ) {
			for( int col = 0; col < getColumnCount(); col++ ) {
				double d = getElement( row, col );
				if( d < 0 || !Util.isZero(d - getElement( col, row ) ) ) { 
					return false;
				}
			}
		}
		return true;
	}
    
	public boolean equals( Object o ) {
		if( !(o instanceof NumericMatrixInterface ) ) {
			return false;
		}
		NumericMatrixInterface m = (NumericMatrixInterface)o;
		if( getRowCount() != m.getRowCount() || getColumnCount() != m.getColumnCount() ) {
			return false;
		}
		for( int i = 0; i < getRowCount(); i++ ) {
			for( int j = 0; j < getColumnCount(); j++ ) {
				if( getElement(i, j) != m.getElement(i, j) ) {
					return false;
				}
			}
		}
		return true;
	}
	
	public String toString() {
		StringBuffer rc = new StringBuffer();
		for( int i = 0; i < getRowCount(); i++ ) {
			for( int j = 0; j < getColumnCount(); j++ ) {
				rc.append(getElement(i, j));
				rc.append("\t");
			}
			rc.append("\n");
		}
		return rc.toString();
	}

	public final ColumnInterface getColumnView( int colIndex ) {
		return getNumericColumnView( colIndex );
	}
	
	public final RowInterface getRowView( int rowIndex ) {
		return getNumericRowView( rowIndex );
	}
	
	public NumericColumnInterface getNumericColumnView( int colIndex ) {
		return new NumericColumn( new ColumnVector(colIndex) );
	}
	
	public NumericRowInterface getNumericRowView( int rowIndex ) {
		return new NumericRow( new RowVector( rowIndex ) );
	}
	
	public void pivotIndexForL() {
		m_bFrontPivot = false;
/*		
		for( int i = 0; i < getRowCount(); i++ ) {
			m_pivots[i] = i;
		}
*/		
	}
	
	public void pivotIndexForU() {
		m_bFrontPivot = true;
/*		
		for( int i = 0; i < getRowCount(); i++ ) {
			m_pivots[i] = getRowCount()-1-i;
		}
*/		
	}
	
	private class ColumnVector extends AbstractNumericVector {
		private int m_col;
		
		public ColumnVector( int col ) {
			m_col = col;
		}
		
		public final Object clone() {
			// this clone create a new copy of the vector
			// this copy references the matrix
			return new NumericVector(this);
		}
		
		public final int getDimension() {
			return getRowCount();
		}
		
		public final double getValue(int index) {
			return getElement(index, m_col);
		}		
		
		public final void setValue(int index, double value) {
			setElement(index, m_col, value);
		}
		
		public final void setValue( NumericVectorInterface v ) {
			if( v.getDimension() != getDimension() ) {
				throw new linearalgebra.exception.SizeMismatchError("Vector sizes do not match. Vector cannot copy values.");
			}
			for( int i = 0; i < getDimension(); i++ ) {
				setValue(i, v.getValue(i));
			}
		}
	}
	
	private class RowVector extends AbstractNumericVector {
		private int m_row;
		
		public RowVector( int row ) {
			m_row = row;
		}
		
		public final Object clone() {
			// this clone create a new copy of the vector
			// this copy references the matrix
			return new NumericVector(this);
		}
		
		public final int getDimension() {
			return getColumnCount();
		}
		
		public final double getValue(int index) {
			return getElement(m_row, index);
		}		
		
		public final void setValue(int index, double value) {
			setElement( m_row, index, value );
		}		
		
		public final void setValue( NumericVectorInterface v ) {
			if( v.getDimension() != getDimension() ) {
				throw new linearalgebra.exception.SizeMismatchError("Vector sizes do not match. Vector cannot copy values.");
			}
			for( int i = 0; i < getDimension(); i++ ) {
				setValue(i, v.getValue(i));
			}
		}
	}

	private static void setDiagonalToOne( AbstractMatrix m ) {
		for( int i = 0; i < m.getRowCount(); i++ ) {
			if( Util.isZero(m.getElement(i,i)) ) {
				m.setElement(i, i, 1.0);
			}
		}
	}
	
	private boolean partialPivotDown( int row, int col, PermutationMatrix p, MatrixInterface l ) {
		double maxValue = Math.abs(getElement(row, col));
		int maxIndex = row;
		for( int i = row+1; i < getRowCount(); i++ ) {
			double tempValue = Math.abs(getElement( i, col ));
			if( tempValue >  maxValue ) {
				maxValue = tempValue;
				maxIndex = i;
			}
		}
		if( row != maxIndex ) {
			swapRows(row, maxIndex);
			p.swapRows(row, maxIndex);
			l.swapRows(row, maxIndex);
		}
		return maxValue > 0.0;	// returns true if there was a partial pivot
	}
	
	private MatrixInterface choleskyDecomposition() {
		Matrix L = (Matrix)factorL(null, false);
		if( isPositiveDefinite() ) {
			for( int i = 0; i < getRowCount(); i++ ) {
				double p = getElement(i, i);
				p = Math.sqrt(p);
				L.getNumericColumnView(i).multiplyEquals(p);
			}
			for( int row = 0; row < L.getRowCount(); row++ ) {
				for( int col = 0; col <= row; col++ ) {
					setElement(col, row, L.getElement(row, col));
				}
			}
		}
		return L;
	}
	
	public MatrixInterface getProjectionMatrix() {
		try {
			NumericMatrixInterface a = this;
			NumericMatrixInterface at = new TransposeWrapper(a);
			NumericMatrixInterface ata = at.product(a);
			// TODO - this inverse should be a cholesky decomposition??
			NumericMatrixInterface atai = (NumericMatrixInterface)ata.factorInverse();
			return product(atai).product(at);
		} catch( linearalgebra.exception.UninvertibleMatrixException e ) {
			throw new Error("ata should be invertable.");
		}
	}	
	
	public MatrixInterface getOrthogonalProjectionMatrix() {
		// I-P
		MatrixInterface p = getProjectionMatrix();
		return new Matrix(p.getRowCount()).minus((NumericMatrixInterface)getProjectionMatrix());
	}

	public MatrixInterface getReflectionMatrix() {
		// I-2P
		NumericMatrixInterface p = (NumericMatrixInterface)getProjectionMatrix();
		p.multiply(2);
		return new Matrix(p.getRowCount()).minus((NumericMatrixInterface)getProjectionMatrix());
	}

	public NumericMatrixInterface minus( NumericMatrixInterface m ) {
		NumericMatrixInterface mm = (NumericMatrixInterface)clone();
		return mm.minusEquals(m);		
	}
	
	public NumericMatrixInterface minusEquals( NumericMatrixInterface m ) {
		if( getRowCount() != m.getRowCount() || getColumnCount() != m.getColumnCount() ) {
			throw new SizeMismatchError("Matrices must have similar dimensions for addition.");
		}
		for( int i = 0; i < getRowCount(); i++ ) {
			for( int j = 0; j < getColumnCount(); j++ ) {
				incrementElement( i, j, -m.getElement(i, j) );
			}
		}
		return this;
	}	

	public boolean isOrthogonal() {
    	for( int i = 0; i < getColumnCount(); i++ ) {
    		VectorInterface c1 = getNumericColumnView(i).getNumericVectorView();
    		double l = c1.getLengthSquared();
    		if( !Util.isEqual(l, 1.0) ) {
    			return false;
    		}
    		for( int j = i+1; j < getColumnCount(); j++ ) {
    			VectorInterface c2 = getNumericColumnView(j).getNumericVectorView();
    			if( !c1.isOrthogonal(c2) ) {
    				return false;
                }
    		}
    	}
    	return true;
	}
}
