package linearalgebra.numeric;

import linearalgebra.PermutationMatrix;
import linearalgebra.VectorInterface;
import linearalgebra.exception.SizeMismatchError;
import linearalgebra.exception.NoPivotError;
import linearalgebra.exception.UnsupportedOperationError;
import linearalgebra.ColumnInterface;
import linearalgebra.RowInterface;
import linearalgebra.MatrixInterface;

public	class		NumericColumn 
		implements	NumericColumnInterface {
			
	private NumericVectorInterface m_vector;
	
	public NumericColumn( int size ) {
		m_vector = new NumericVector(size);
	}
	
	public NumericColumn( NumericVectorInterface proxy ) {
		m_vector = proxy;
	}
	
	public NumericColumn( double[] copy ) {
		m_vector = new NumericVector(copy);
	}
	
	public NumericColumn( NumericMatrixInterface copy, int column ) {
		m_vector = new NumericVector( copy.getNumericColumnView(column).getVectorView() );
	}
	
	public NumericColumn( NumericColumn copy ) {
		this( (NumericVector)(copy.m_vector.clone()) );
	}
	
	public Object clone() {
		try {
			Object rc = super.clone();
			((NumericColumn)rc).m_vector = (NumericVector)m_vector.clone();
			return rc;
		} catch( CloneNotSupportedException e ) {
			throw new InternalError("CloneNotSupported unexpectedly caught in NumericColumn.clone()");
		}
	}
	
	public VectorInterface getVectorView() {
		return getNumericVectorView();
	}
	
	public NumericVectorInterface getNumericVectorView() {
		return m_vector;
	}
	
	public int getRowCount() {
		return m_vector.getDimension();
	}
	
	public int getDimension() {
		return getRowCount();
	}
	
	public void multiplyRow( int index, double scalar ) {
		m_vector.scaleDimension(index, scalar);
	}
	
	public void combineRows( int srcRow, int dstRow, double scalar ) {
		m_vector.linearCombinationDimension(srcRow, dstRow, scalar);
	}
	
	public double getRowValue( int row ) {
		return m_vector.getValue(row);
	}

	public void swapRows( int row1, int row2 ) {
		m_vector.swapDimensions( row1, row2 );
	}	

	public void swapRows( PermutationMatrix p ) {
		NumericVector temp = new NumericVector(m_vector.getDimension());
		for( int i = 0; i < getRowCount(); i++ ) {
			temp.setValue(i, m_vector.getValue(p.getRowTranspose(i)));
		}
		m_vector = temp;
	}	

	public boolean hasRowPivot( int row ){
		if( m_vector.getValue(row) != 0 ) {
			return true;
		}
		return false;
	}
	
	public boolean hasColumnPivot( int col ){
		for( int i = 0; i < getRowCount(); i++ ) {
			if( m_vector.getValue(i) != 0 ) {
				return true;
			}
		}
		return false;
	}
	
	public int getRowLeftPivotIndex( int row ){
		if( m_vector.getValue(row) != 0.0 ) {
			return 0;
		}
		throw new linearalgebra.exception.NoPivotError();
	}
	
	public int getRowRightPivotIndex( int row ){
		if( m_vector.getValue(row) != 0.0 ) {
			return 0;
		}
		throw new linearalgebra.exception.NoPivotError();
	}
	
	public int getColumnTopPivotIndex( int col ) {
		for( int i = 0; i < getRowCount(); i++ ) {
			if( m_vector.getValue(i) != 0 ) {
				return i;
			}
		}
		throw new NoPivotError();
	}
	
	public int getColumnBottomPivotIndex( int col ){
		for( int i = 0; i < getRowCount(); i++ ) {
			if( m_vector.getValue(i) != 0 ) {
				return i;
			}
		}
		throw new NoPivotError();
	}
	
	public int getColumnCount() {
		return 1;
	}
	
	// operations for eliminations
	public double getElement( int row, int col ) {
		if( col != 0 ) {
			throw new ArrayIndexOutOfBoundsException();
		}
		return getRowValue(row);
	}
		
	// arithmetic operations
	
	public NumericMatrixInterface plus( NumericMatrixInterface m ) {
		NumericColumn rc = new NumericColumn(this);
		return rc.plusEquals(m);
	}
	
	public NumericMatrixInterface plusEquals( NumericMatrixInterface m ) {
		if( getRowCount() != m.getRowCount() || 1 != m.getColumnCount() ) {
			throw new SizeMismatchError("Matrices must have similar dimensions for addition.");
		}
		NumericVectorInterface vector2 = m.getNumericColumnView(0).getNumericVectorView();
		m_vector.linearCombinationEquals(vector2, 1.0);
		return this;
	}
	
	public NumericMatrixInterface minus( NumericMatrixInterface m ) {
		NumericColumn rc = new NumericColumn(this);
		return rc.minusEquals(m);
	}
	
	public NumericMatrixInterface minusEquals( NumericMatrixInterface m ) {
		if( getRowCount() != m.getRowCount() || 1 != m.getColumnCount() ) {
			throw new SizeMismatchError("Matrices must have similar dimensions for subtraction.");
		}
		NumericVectorInterface vector2 = m.getNumericColumnView(0).getNumericVectorView();
		m_vector.linearCombinationEquals(vector2, -1.0);
		return this;
	}
	
	public NumericMatrixInterface multiply( double scalar ) {
		NumericColumn rc = new NumericColumn(this);
		return rc.multiplyEquals(scalar);
	}
	
	public NumericMatrixInterface multiplyEquals( double scalar ) {
		m_vector.scale(scalar);
		return this;
	}
	
	public NumericMatrixInterface product( NumericMatrixInterface right ) {
		if( getColumnCount() != right.getRowCount() ) {
			throw new SizeMismatchError("Row count of first matrix must equal column count of second matrix for cross product.");
		}
		Matrix rc = new Matrix( getRowCount(), right.getColumnCount() );
		
		for( int row = 0; row < rc.getRowCount(); row++ ) {
			VectorInterface rowVector = right.getNumericRowView(row).getVectorView();
			for( int col = 0; col < rc.getColumnCount(); col++ ) {
				rc.setElement(row, col, m_vector.getValue(row)*rowVector.getValue(col));
			}
		}
		
		return rc;
	}
	
	public NumericMatrixInterface power( int power ) {
		NumericMatrixInterface rc = this;
		for( int i = 1; i < power; i++ ) {
			rc = rc.product(this);
		}
		return rc;
	}

	public boolean equals( Object o ) {
		boolean rc = false;
		if( o instanceof NumericMatrixInterface ) {
			NumericMatrixInterface m = (NumericMatrixInterface)o;
			if( getRowCount() == m.getRowCount() && getColumnCount() == m.getColumnCount() ) {
				if( m_vector.equals( m.getNumericColumnView(0).getVectorView() ) ) {
					rc = true;
				}
			}
		}
		return rc;
	}
	
	public ColumnInterface getColumnView( int col ) {
		return getNumericColumnView(col);
	}
	
	public RowInterface getRowView( int row ) {
		return getNumericRowView(row);
	}
	
	public NumericColumnInterface getNumericColumnView( int col ) {
		if( col != 0 ) { 
			throw new ArrayIndexOutOfBoundsException(); 
		}
		return this;
	}
	
	public NumericRowInterface getNumericRowView( int row ) {
		NumericVectorInterface v = new SingleElementVector(getNumericVectorView(), row);
		return new NumericRow( v );
	}
	
	public String toString() {
		StringBuffer rc = new StringBuffer();
		for( int row = 0; row < getRowCount(); row++ ) {
			rc.append(getRowValue(row)+"\n");
		}
		return rc.toString();
	}
	
    public MatrixInterface factorL(PermutationMatrix p) {
        return factorL(p, true);
    }

	public MatrixInterface factorL(PermutationMatrix p, boolean partialPivot) {
		Matrix rc = new Matrix( getDimension() );	// creates an identity matrix
		
		int pivot_index = 0;
		while( getElement(pivot_index, 0) == 0.0 ) {
			++pivot_index;
			if( pivot_index >= getDimension() ) {
				return rc;
			}
		}
		double pivot = getElement( pivot_index, 0 );
		for( int row = 0; row < rc.getRowCount(); row++ ) {
			rc.setElement(row, 0, getElement(row,0)/pivot);
		}
		m_vector.setValue(0, pivot);
		for( int row = 1; row < rc.getRowCount(); row++ ) {
			m_vector.setValue(row, 0.0);
		}

		return rc;
	}
	
	public MatrixInterface factorU(PermutationMatrix p) {
		Matrix rc = new Matrix( new double[][] {{1.0}} );
		return rc;
	}

	public MatrixInterface factorInverse() throws linearalgebra.exception.UninvertibleMatrixException {
		throw new linearalgebra.exception.UninvertibleMatrixException();
	}
	
	public MatrixInterface getTranspose() {
		return new NumericRow( (NumericVector)m_vector.clone() );
	}
	
	public boolean isIdentity() {
		return ( getDimension() == 1 ) && ( getRowValue(0) == 1.0 );
	}
	
	public boolean isUpperRightTriangular() {
		return true;
	}
	
	public boolean isLowerLeftTriangular() {
		boolean rc = true;
		for( int i = 1; i < getDimension(); i++ ) {
			if( getRowValue(i) != 0.0 ) {
				rc = false;
				break;
			}
		}
		return rc;
	}
	
	public boolean isSymmetric() {
		return false;
	}

	public boolean isPositiveDefinite() {
		return false;
	}

	
    public void incrementElement(int row, int col, double value) {
    	if( col == 0 ) {
    		m_vector.setValue( row, m_vector.getValue(col)+value );
    	} else {
    		throw new ArrayIndexOutOfBoundsException();
    	}
    }
    
    public void scaleElement(int row, int col, double scalar) {
    	if( col == 0 ) {
    		m_vector.scaleDimension(row, scalar);
    	} else {
    		throw new ArrayIndexOutOfBoundsException();
    	}
    }
    
    public void setElement(int row, int col, double element) {
    	if( col == 0 ) {
    		m_vector.setValue(row, element);
    	} else {
    		throw new ArrayIndexOutOfBoundsException();
    	}
    }
    
    public MatrixInterface getProjectionMatrix() {
    	throw new UnsupportedOperationError();
    }
    
    public MatrixInterface getOrthogonalProjectionMatrix() {
    	throw new UnsupportedOperationError();
    }
    
    public MatrixInterface getReflectionMatrix() {
    	throw new UnsupportedOperationError();
    }
    
    public boolean isIdempotent() {
    	throw new UnsupportedOperationError();
    }

	public boolean isOrthogonal() {
		return false;
	}
}